clear variables
close all
clc

% load the simulation data
simdata = ADSimportSimData('Oscillator.mat');

% combine the different measured voltages as a system with one input and three outputs
data = reshape([simdata.sim1_AC.Vint;simdata.sim1_AC.Vout;simdata.sim1_AC.Vres],3,1,[]);
freq = simdata.sim1_AC.freq;

% construct an FRM object with the simulation data.
% we set the normalisation to lowpass, because we know the impedance data close to DC
Z = FRM(data,'Freq',freq,'Normalisation',LowpassNormalisation(freq));

% perform the projection to determine the stability of the circuit solution
% the data is noiseless, so we use a rational interpolation to obtain the lowest interpolation error
PROJ = PisaProject(Z,'Interpolation',RationalInterpolation);

% plot the result
figure(1)
plot(Z,'Marker','.','Color',[0 0.4 0]);
plot(PROJ.stable,'Marker','.','Color','b');
plot(PROJ.unstable,'Marker','.','Color','r');
xlim([0 max(freq)])
ylim([-20 60])

% estimate the location of the unstable pole
[poles,singularvalues] = PisaEstimate(Z,'Projection',PROJ,'order',2);

fprintf('poles estimated at: %g GHz\n',abs(imag(poles(1)))/2/pi/1e9);
fprintf(' with a damping of %2.2g \n',cos(pi-angle(poles(1))));

% also plot the singular values of the Hankel matrix 
% to show that there are indeed only two dominant singular values
figure(3)
plot(log10(singularvalues/singularvalues(1)),'o-','linewidth',2);
xlabel('i')
ylabel('log_{10}(\sigma_i / \sigma_1)');
xlim([1 20])
grid on
